@@ -6,10 +6,13 @@
 #include "ypopup.h"
 #include "ystring.h"
 
+#include <regex.h>
+
 class YMenu;
 class YInputLine;
 class YInputMenu;
 class YWideString;
+struct tCandCollector;
 
 class YInputListener {
 public:
@@ -55,6 +58,7 @@ public:
     bool move(unsigned pos, bool extend);
     bool hasSelection() const { return curPos != markPos; }
     void replaceSelection(const char* str, int len);
+    void replaceSelection(wchar_t* str, int len);
     bool deleteSelection();
     bool deleteNextChar();
     bool deletePreviousChar();
@@ -68,7 +72,7 @@ public:
     void unselectAll();
     bool cutSelection();
     bool copySelection();
-    void complete();
+    void complete(bool previewOnly = false);
 
 protected:
     virtual bool handleTimer(YTimer *timer);
@@ -80,6 +84,7 @@ private:
     unsigned offsetToPos(int offset);
     static mstring completeVariable(mstring var);
     static mstring completeUsername(mstring user);
+    int getWCharFromEvent(const XKeyEvent& key, wchar_t* s, int maxLen);
 
     YWideString fText;
     unsigned markPos;
@@ -93,6 +98,7 @@ private:
     unsigned fKeyPressed;
     YInputListener* fListener;
 
+    XIC inputContext;
     YFont inputFont;
     YColorName inputBg;
     YColorName inputFg;
@@ -100,6 +106,8 @@ private:
     YColorName inputSelectionFg;
     lazy<YTimer> cursorBlinkTimer;
     lazy<YInputMenu> inputMenu;
+    regex_t* prefixRegex;
+    osmart<tCandCollector> lastSeenCandidates;
 
 private: // not-used
     YInputLine(const YInputLine &);
